; VGA Test Pattern Generator (H/W Debug Pattern)
; Ikagawa/Ikagawa Lab.
; 1999.10.09 Ver.1.00

; Test pattern for DAC resistor value adjustment
; RGB line termination = 75 ohm
;
; video signal (RGB) pattern
;                          _______
;                         |       |
;                  _______|       |
;                 |               |
;          _______|               |
;         |                       |
;  _______|                       |_______
;
;  0.000V  0.233V  0.467V  0.700V  0.000V
;
;|<--------------- 1 H ----------------->|
;


        .device AT90S1200

        .equ    INT0addr        = $001
        .equ    OVF0addr        = $002
        .equ    ACIaddr         = $003

        .equ    PORTB           = $18
        .equ    DDRB            = $17
        .equ    PINB            = $16
        .equ    PORTD           = $12
        .equ    DDRD            = $11
        .equ    PIND            = $10

        .cseg

        .org $0000
        rjmp    RESET                   ; Reset handler

        .org INT0addr
        reti                            ; External interrupt handler

        .org OVF0addr
        reti                            ; Timer0 overflow handler

        .org ACIaddr
        reti                            ; Analog comparator handler


RESET:
        ldi     r16,$FF
        out     DDRB,r16                ; portb = output

        ldi     r16,$FF
        out     DDRD,r16                ; portd = output

        ldi     r16,$03
        out     PORTB,r16               ; vsync = 1, hsync = 1

        ldi     r16,$00
        out     PORTD,r16               ; video = 00_00_00


MAIN:
        ldi     r17,$ff                 ; 1
activelineloop0:
        rcall   ACTIVELINE              ; 378
        dec     r17                     ; 1
        brne    activelineloop0         ; 1/2

        ldi     r17,$e1                 ; 1
activelineloop1:
        rcall   ACTIVELINE              ; 378
        dec     r17                     ; 1
        brne    activelineloop1         ; 1/2

        ldi     r17,$09                 ; 1
blanklineloop0:
        rcall   BLANKLINE               ; 378
        dec     r17                     ; 1
        brne    blanklineloop0          ; 1/2

        ldi     r17,$19                 ; 1
vsynclineloop0:
        rcall   VSYNCLINE               ; 378
        dec     r17                     ; 1
        brne    vsynclineloop0          ; 1/2

        ldi     r17,$0a                 ; 1
blanklineloop1:
        rcall   BLANKLINE               ; 378
        dec     r17                     ; 1
        brne    blanklineloop1          ; 1/2

        nop                             ; 1
        rcall   BLANKLINE               ; 378
        rjmp    MAIN                    ; 2






;------------------------------------------------------------------------------
; rcall(3) + 371 + ret (4) = 378
;
ACTIVELINE:
        ldi     r16,$00                 ; 1     00_00_00
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$00                 ; 1     00_00_00
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$15                 ; 1     01_01_01
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$15                 ; 1     01_01_01
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$2a                 ; 1     10_10_10
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$2a                 ; 1     10_10_10
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$3f                 ; 1     11_11_11
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$3f                 ; 1     11_11_11
        out     PORTD,r16               ; 1
        rcall   WAIT36                  ; 36

        ldi     r16,$00                 ; 1     00_00_00
        out     PORTD,r16               ; 1

        nop                             ; 1
        nop                             ; 1
        nop                             ; 1
        nop                             ; 1
        nop                             ; 1
        nop                             ; 1

        ldi     r16,$02                 ; 1
        out     PORTB,r16               ; 1     vsync = 1, hsync = 0
        rcall   WAIT44                  ; 44

        ldi     r16,$03                 ; 1
        out     PORTB,r16               ; 1     vsync = 1, hsync = 1
        rcall   WAIT11                  ; 11

        ret                             ; 4
;------------------------------------------------------------------------------
BLANKLINE:
        ldi     r16,$00                 ; 1     00_00_00
        out     PORTD,r16               ; 1
        rcall   WAIT310                 ; 310

        ldi     r16,$02                 ; 1
        out     PORTB,r16               ; 1     vsync = 1, hsync = 0
        rcall   WAIT44                  ; 44

        ldi     r16,$03                 ; 1
        out     PORTB,r16               ; 1     vsync = 1, hsync = 1
        rcall   WAIT11                  ; 11

        ret                             ; 4
;------------------------------------------------------------------------------
VSYNCLINE:
        ldi     r16,$00                 ; 1     00_00_00
        out     PORTD,r16               ; 1
        rcall   WAIT310                 ; 310

        ldi     r16,$00                 ; 1
        out     PORTB,r16               ; 1     vsync = 0, hsync = 0
        rcall   WAIT44                  ; 44

        ldi     r16,$01                 ; 1
        out     PORTB,r16               ; 1     vsync = 0, hsync = 1
        rcall   WAIT11                  ; 11

        ret                             ; 4
;------------------------------------------------------------------------------
WAIT11:
        nop                             ; 1
        nop                             ; 1
        nop                             ; 1
        nop                             ; 1

        ret                             ; 4
;------------------------------------------------------------------------------
WAIT36:
        ldi     r16,$09                 ; 1
wait36loop:
        dec     r16                     ; 1
        brne    wait36loop              ; 1/2
        nop                             ; 1
        nop                             ; 1

        ret                             ; 4
;------------------------------------------------------------------------------
WAIT44:
        ldi     r16,$0c                 ; 1
wait44loop:
        dec     r16                     ; 1
        brne    wait44loop              ; 1/2
        nop                             ; 1

        ret                             ; 4
;------------------------------------------------------------------------------
WAIT310:
        ldi     r16,$65                 ; 1
wait310loop:
        dec     r16                     ; 1
        brne    wait310loop             ; 1/2

        ret                             ; 4
;------------------------------------------------------------------------------
